
console.log('CME-----------------------> common-utils');

// ---------------------------------------------------- //
// ----------------- Element creation ----------------- //
// ---------------------------------------------------- //

// mark each created element to clear them if need
var nbOfCustomItem = 0;
const CUSTOM_ITEM_CLASS = 'custom-roll20-ext-mod';
function createCustomElement(type, id) {
	const e = document.createElement(type);
	e.className = CUSTOM_ITEM_CLASS;
	nbOfCustomItem++;
	if (id != null && id) {
		e.id = id;
	}
	return e;
}
function createCustomObject(type, id) {
	return $(createCustomElement(type, id));
}

// waiting an element creation in the DOM
function waitElementExist(id, callback, time) {
	if(document.getElementById(id)) {
		console.log('THEME: "'+id+'" ok');
		callback();
	} else {
		console.log('THEME: waiting "'+id+'" ...');
		setTimeout(() => { waitElementExist(id, callback); }, time ? time : 500);
	}
}

// ----------------------------------------------------------- //
// ----------------- Data loaded observer ----------------- //
// ----------------------------------------------------------- //
var isLoaded = $('#loading-overlay').length > 0 && ($('#loading-overlay').css('display') == 'none');
function doWhenLoad(callback) {
	const interval = 500;
	new Promise((callback, err) => {
		const fct = () => {
			isLoaded = isLoaded || ($('#loading-overlay').length > 0 && ($('#loading-overlay').css('display') == 'none'));
			if(!isLoaded) {
				setTimeout(fct, interval);
			} else {
				callback();
			}
		}
		setTimeout(fct, interval);
	}).then(callback, null);
}

// ----------------------------------------------------------- //
// ----------------- Child creation observer ----------------- //
// ----------------------------------------------------------- //
function doWhenChildCreated(root, checkFct, callback) {
	new MutationObserver((mutationsList) => {
		for(var mutation of mutationsList) {
			// parse new node added into the chat
			for (var node of mutation.addedNodes) {
				// check init to avoid auto-play media from old msg
				if (node.nodeType === Node.ELEMENT_NODE) {
					var myNode = $(node);
					console.log('doWhenChildCreated ?');
					if (checkFct(myNode)) {
						console.log('doWhenChildCreated : OK');
						callback(myNode);
					} else {
						console.log('doWhenChildCreated : Check fail : ' + myNode.attr('class'));
					}
				}
			}
		}
	}).observe(root.get(0), {
		childList: true
	});
}


// ---------------------------------------------------------------- //
// ----------------- css style 'display' observer ----------------- //
// ---------------------------------------------------------------- //
function doWhenDislayChange(element, callback) {
	console.log('doWhenDislayChange !');
	var previous = element.css('display');
	new window.MutationObserver((mutationsList) => {
		var current = element.css('display');
		if (current != previous) {
			console.log('doWhenDislayChange > display change !!!');
			previous = current;
			callback(element);
		}
	}).observe(element.get(0), {
	  attributes: true,
	  attributeFilter: ['style']
	});
}

// ------------------------------------------------- //
// ----------------- Drag and drop ----------------- //
// ------------------------------------------------- //

var droppableElt = null;
var dropDestElt = null;
var droppingAction = null;
var dropOutManager = null;

function setAsDraggable(elmnt, customHandler, onDragStart, onDropOut, onSimpleClick) {

	console.log('elmnt :: ' + elmnt.className);
	// to manage the drop cancel
	var beforeMoveX = 0, beforeMoveY = 0;

	// to manage the dynamic drag move
	var offsetX = 0, offsetY = 0, oldX = 0, oldY = 0, originX = 0, originY = 0;
	var dragging=false, draggingDone=false;

	var handler = customHandler ? customHandler : elmnt;
	handler.onmousedown = dragMouseDown;
	$(handler).click(function() {
		console.log('click !!! ' + draggingDone);
		// simple click callback (when no drag)
		if (!draggingDone && onSimpleClick) {
			console.log('simple click !!!');
			onSimpleClick();
		}
	});

	function dragMouseDown(e) {
		dragging=false;
		draggingDone=false;
		e = e || window.event;
		e.preventDefault();
		if(onDragStart) {
			onDragStart();
		}

		// get the mouse cursor position at startup:
		originX = elmnt.offsetLeft;
		originY = elmnt.offsetTop;
		oldX = e.clientX;
		oldY = e.clientY;
		document.onmouseup = closeDragElement;
		// call a function whenever the cursor moves:
		document.onmousemove = elementDrag;
		//to manage drop
		document.body.classList.add('dragging');
		droppableElt = elmnt;
		dropDestElt = null;
		console.log('drag:: ' + droppableElt.id);
	}

	function elementDrag(e) {
		dragging = true;
		e = e || window.event;
		e.preventDefault();
		// calculate the new cursor position:
		offsetX = e.clientX - oldX;
		offsetY = e.clientY - oldY;
		originX = originX + offsetX;
		originY = originY + offsetY;
		oldX = e.clientX;
		oldY = e.clientY;
		// set the element's new position:
		elmnt.style.left = originX + "px";
		elmnt.style.top = originY + "px";

	}

	function closeDragElement() {
		draggingDone = dragging;
		dragging = false;
		/* stop moving when mouse button is released:*/
		document.onmouseup = null;
		document.onmousemove = null;
		document.body.classList.remove('dragging');
		if (dropDestElt) {
			// perform drop
			console.log('dropping...');
			if (droppingAction) {
				console.log('dropping action...');
				console.log('drop:: ' + droppableElt.id);
				droppingAction(droppableElt, onDropOut);
			}
		} else {
			console.log('no drop');
		}
		droppableElt = null;
		droppingAction = null;
	}
}

function setAsDropContainer(container, droppingFct) {
	container.onmouseover = () => {
		if(droppableElt) {
			droppingAction = droppingFct;
			dropDestElt = container;
		}
	};
	container.onmouseout = () => {
		if(droppableElt && dropDestElt===container) {
			droppingAction = null;
			dropDestElt = null;
		}
	};
}